/**
 * CopyRight (C) 2005 Bam Rescue Simulation Team.All rights resserved.
 *
 * Hamid Mohammadi <hmohammadi110@gmail.com>
 * Hamed Shahbazi  <demahopilot@yahoo.com>
 * Behrooz Shahgholi <shahgholi@eng.ui.ac.ir>
 *
 */


package bam;

import java.net.*;
import java.util.*;
import yab.agent.*;
import yab.agent.object.*;
import yab.agent.DisasterSpace.*;

public class AmbulanceTeamAgent extends AbstractAmbulanceTeamAgent {

    //Member Fields
    private ArrayList m_unvisitedRoads,
            reportedInj = new ArrayList(),
            brdReports = new ArrayList(),
            visitedRaods = new ArrayList(),
            civilianBuildings = new ArrayList(),
            reportedAgents = new ArrayList();

    private int searchTime = 0;
    private recordHuman bestHumanoid = new recordHuman();



    protected static final Property
            DAMAGE_PRP = Property.get("Humanoid", "damage"),
            BURIEDNESS_PRP = Property.get("Humanoid", "buriedness"),
            HEALTH_PRP = Property.get("Humanoid", "hp"),
            passLine = Property.get("Road", "passableLines"),
            POSITION_PRP = Property.get("MovingObject", "position"),
            FIERYNESS_PRP = Property.get("Building", "fieryness"),
            HASH_PRP = Property.get("Humanoid", "id");

    protected static final Condition
            IS_NOT_BURNING_CND = FIERYNESS_PRP.lt(1).or(FIERYNESS_PRP.gt(3)),
            passCnd = passLine.lt(1),
            IS_BURNING_CND = FIERYNESS_PRP.gte(1).and(FIERYNESS_PRP.lte(3)),
            IS_PUT_OUT_CND = FIERYNESS_PRP.gte(5).and(FIERYNESS_PRP.lte(7)),
            HAS_BEEN_SEEN_CND = Property.get("RealObject", "hasBeenSeen").equal(Boolean.TRUE),
            HAS_NOT_CIVILIAN = Property.get("Building", "havecivilian").equal(Boolean.FALSE),
            HAS_NOT_BEEN_SEEN_CND = Property.get("RealObject", "hasBeenSeen").equal(Boolean.FALSE);



    //Methods

    public AmbulanceTeamAgent(InetAddress address, int port) {
        super(address, port);
        Initializze();
   }


    protected void Initializze() {
        m_unvisitedRoads = new ArrayList(world.buildings);
    }

    //returns motion less position of self. This is only a gift!
    protected MotionlessObject mlpos() {
        return self().motionlessPosition();
    }

    private ArrayList reportedTillNow = new ArrayList();
    private ArrayList reportedBlockadeOthers = new ArrayList();

    protected void prepareForAct() {
        super.prepareForAct();
        if (mlpos() instanceof Building) {
            ArrayList deleteTemp1 = new ArrayList();
            deleteTemp1.add(mlpos());
            reportedAgents.removeAll(deleteTemp1);

            //remove reportedinj
            ArrayList deleteTemp = new ArrayList();
            for (Iterator itrt = reportedInj.iterator(); itrt.hasNext(); ) {
                recordHuman damage = (recordHuman) itrt.next();
                if (damage.mlpos.equals(mlpos())) {
                    deleteTemp.add(damage);
                }
            }
            reportedInj.removeAll(deleteTemp);
        }
    }

    private void search() {
        for (Iterator itr = world.nearHumanoid.iterator(); itr.hasNext(); ) {
            Humanoid htemp = (Humanoid) itr.next();
            if (htemp instanceof Civilian &&
                !(htemp.motionlessPosition() instanceof Refuge) &&
                !htemp.loaded) {
                if (htemp.damage() > 0) {
                    recordHuman rcH = new recordHuman();
                    rcH.buridness = htemp.buriedness();
                    rcH.damage = htemp.damage();
                    rcH.hp = htemp.hp();
                    rcH.id = htemp.id;
                    rcH.self = htemp;
                    rcH.mlpos = htemp.motionlessPosition();
                    if (!reportedInj.contains(rcH)) {
                        reportedInj.add(rcH);
                    } else {
                        reportedTillNow.add(rcH);
                        reportedInj.remove(rcH);
                        reportedInj.add(0, rcH);
                    }
                }

            }
        }
    }


    private void checkSituation() throws ActionCommandException {
        if (self().buriedness() > 0 && time() > 5) {
            tell("pfh " + self().hp() + " " + self().damage() + " " +
                 self().buriedness() + " " + self().id + " " +
                 self().motionlessPosition().id);
        }

        if (self().damage() != 0) {
            move(world.refuges);
        }
    }


    private void checkBestHumanoid() throws ActionCommandException {
        if (bestHumanoid.isSelected()) {

            if (bestHumanoid.self instanceof Civilian && reportedAgents.size() >= 1) {
                bestHumanoid.clearSelected();
            }

            if ((world.time() - searchTime) > 16 && (mlpos() instanceof Road)) {
                bestHumanoid.clearSelected();
            }

            if (((Building) bestHumanoid.mlpos).fieryness() > 0 &&
                ((Building) bestHumanoid.mlpos).fieryness() <= 3) {
                bestHumanoid.clearSelected();

                if (reportedAgents.contains(bestHumanoid)) {
                    reportedAgents.remove(bestHumanoid);
                } else if (reportedInj.contains(bestHumanoid)) {
                    reportedInj.remove(bestHumanoid);
                }
            }

            if (bestHumanoid.self.hp() == 0) {
                bestHumanoid.clearSelected();

                if (reportedAgents.contains(bestHumanoid)) {
                    reportedAgents.remove(bestHumanoid);
                } else if (reportedInj.contains(bestHumanoid)) {
                    reportedInj.remove(bestHumanoid);
                }

            }
        }

    }

    private void rescueorLoad() throws ActionCommandException {
        if (isLoadingInjured()) {

            carry();
        }

        checkBestHumanoid();

        if (bestHumanoid.isSelected() &&
            bestHumanoid.mlpos() != null &&
            bestHumanoid.mlpos().equals(mlpos())
            && !(self().motionlessPosition() instanceof Road)) {
            if (bestHumanoid.self.buriedness() > 0 &&
                bestHumanoid.self.hp() > 0) {
                rescue(bestHumanoid.self);
            }

            else {
                bestHumanoid.clearSelected();
                if (reportedAgents.contains(bestHumanoid)) {
                    reportedAgents.remove(bestHumanoid);
                } else if (reportedInj.contains(bestHumanoid)) {
                    reportedTillNow.add(bestHumanoid);
                    reportedInj.remove(bestHumanoid);
                    tell("res " + bestHumanoid.id);
                }
            }

            if (bestHumanoid.self instanceof Civilian &&
                bestHumanoid.self.buriedness() == 0 &&
                bestHumanoid.self.hp() > 0 &&
                world.nearHumanoid.contains(bestHumanoid.self)) {

                if ((BURIEDNESS_PRP.eq(0).and(POSITION_PRP.eq(self().
                        motionlessPosition())).extract(world.nearHumanoid).
                     contains(bestHumanoid.self))) {
                    if (reportedAgents.isEmpty() && !reportedInj.isEmpty()) {
                        searchTheInjured(reportedInj);
                    }
                    load((Humanoid) DAMAGE_PRP.max(BURIEDNESS_PRP.eq(0).and(
                            POSITION_PRP.eq(self().motionlessPosition())).
                            extract(world.nearHumanoid)));
                }
            }
        }

    }


    private void correctSearchArea() {
        ArrayList newArray = IS_BURNING_CND.extract(m_unvisitedRoads);
        if (newArray != null) {
            m_unvisitedRoads.removeAll(newArray);
        }
        newArray = IS_PUT_OUT_CND.extract(m_unvisitedRoads);
        if (newArray != null) {
            m_unvisitedRoads.removeAll(newArray);
        }
    }

    private void addCivilianBuilding() {
        for (Iterator it = world.nearHumanoid.iterator(); it.hasNext(); ) {
            Humanoid tempHuman = (Humanoid) it.next();
            if (tempHuman != null) {
                if (tempHuman.position() instanceof MotionlessObject) {
                    if (tempHuman.motionlessPosition() instanceof Building &&
                        !(tempHuman.motionlessPosition() instanceof Refuge) &&
                        tempHuman.hp() != 0) {
                        if (!civilianBuildings.contains((Building) tempHuman.
                                motionlessPosition())) {
                            civilianBuildings.add((Building) tempHuman.
                                                  motionlessPosition());
                        }
                        System.out.println("time=" + world.time() + ",id=" +
                                           tempHuman.id + ",hp=" + tempHuman.hp() +
                                           ",burridness=" +
                                           tempHuman.buriedness() + ",damage=" +
                                           tempHuman.damage());

                    }
                }
            }
        }
    }

    protected void act() throws ActionCommandException {

        m_unvisitedRoads.removeAll(visitedRaods);
        checkSituation();
        reportBlockades();

        checkBloackade();

        rescueorLoad();

        searchAction();

        randomMove();
    }


    private void searchAction() throws ActionCommandException {

        search();
        finBestPrio();
    }

    protected void finBestPrio() throws ActionCommandException {
        if (bestHumanoid.isSelected()) {
            ArrayList moveArray = new ArrayList();
            moveArray.add(bestHumanoid.mlpos);
            move(moveArray);
        } else if (!reportedAgents.isEmpty()) {
            searchTheInjured(reportedAgents);
            ArrayList moveArray0 = new ArrayList();
            moveArray0.add(bestHumanoid.mlpos);
            move(moveArray0);
        } else if (!reportedInj.isEmpty()) {
            searchTheInjured(reportedInj);
            ArrayList moveArray1 = new ArrayList();
            if (bestHumanoid.isSelected()) {
                moveArray1.add(bestHumanoid.mlpos());
            }
            if (!moveArray1.isEmpty()) {
                move(moveArray1);
            }
        } else if (!world.nearHumanoid.isEmpty() &&
                   (reportedInj.size() == 0 || time() < 30)) {
            int maxprio = 100;
            recordHuman bestHuman = new recordHuman();
            bestHuman.hp = ((Humanoid) world.nearHumanoid.get(0)).hp();
            bestHuman.damage = ((Humanoid) world.nearHumanoid.get(0)).damage();
            bestHuman.buridness = ((Humanoid) world.nearHumanoid.get(0)).
                                  buriedness();
            bestHuman.time = ((Humanoid) world.nearHumanoid.get(0)).time();
            bestHuman.id = ((Humanoid) world.nearHumanoid.get(0)).id;
            bestHuman.self = (Humanoid) world.get(bestHuman.id);

            bestHuman.mlpos = ((Humanoid) world.nearHumanoid.get(0)).
                              motionlessPosition();
            maxprio = bestHuman.priority(world.time(),
                                         world.ambulanceTeams.size());

            Iterator itr = world.nearHumanoid.iterator();
            itr.next();
            recordHuman bestHuman1 = new recordHuman();
            for (; itr.hasNext(); ) {
                Humanoid temp = (Humanoid) itr.next();
                if (temp.buriedness() > 0 && temp.hp() > 0 &&
                    !(brdReports.contains(temp))) {
                    bestHuman1.hp = temp.hp();
                    bestHuman1.damage = temp.damage();
                    bestHuman1.buridness = temp.buriedness();
                    bestHuman1.time = temp.time();
                    bestHuman1.mlpos = temp.motionlessPosition();
                    bestHuman1.id = temp.id;
                    bestHuman1.self = (Humanoid) world.get(bestHuman1.id);
                    if (bestHuman1.priority(world.time(),
                                            world.ambulanceTeams.size()) <
                        maxprio &&
                        bestHuman1.mlpos() != null) {
                        bestHuman = bestHuman1;
                        maxprio = bestHuman1.priority(world.time(),
                                world.ambulanceTeams.size());
                    }
                }
            }

            if (bestHuman.buridness > 0 &&
                bestHuman.hp > 0 &&
                bestHuman != null &&
                bestHuman.mlpos() != null &&
                bestHuman.self != null) {
                ArrayList tempList = new ArrayList();
                if (!brdReports.contains(bestHuman.self)) {
                    brdReports.add(bestHuman.self);
                    for (Iterator itrotr = reportedInj.iterator();
                                           itrotr.hasNext(); ) {
                        recordHuman temp = (recordHuman) itrotr.next();
                        if (temp.self == null && temp.id != 0) {
                            temp.self = (Humanoid) world.get(temp.id);
                        }
                        if ((temp).self.equals(bestHuman.self)) {
                            tempList.add(temp);
                        }
                    }

                    reportedInj.removeAll(tempList);
                    reportedInj.add(bestHuman);
                    reportedTillNow.add(tempList);
                    reportedTillNow.add(tempList);
                    tell("c " + bestHuman.hp + " " + bestHuman.damage + " " +
                         bestHuman.buridness + " " + bestHuman.self.id + " " +
                         bestHuman.mlpos().id);
                } else {
                    brdReports.remove(bestHuman.self);
                    brdReports.add(bestHuman.self);
                }
            }

        }

    }

    ArrayList civilianBuilding = new ArrayList();


    protected void randomMove() throws ActionCommandException {
        if (bestHumanoid.hp == 0) {
            bestHumanoid.clearSelected();
        }
        if (!m_unvisitedRoads.isEmpty()) {
            ArrayList selectedRoad = new ArrayList();
            boolean finish = false;
            for (int i = 70000; !finish; i += 10000) {

                boolean mty = false;
                for (Iterator it = m_unvisitedRoads.iterator();
                                   it.hasNext(); ) {
                    Building tempBuid;
                    tempBuid = (Building) it.next();
                    if (self().distance(tempBuid) < i &&
                        tempBuid.isUnburned()) {
                        selectedRoad.add(tempBuid);
                        visitedRaods.add(tempBuid);
                        if (selectedRoad != null) {
                            finish = true;
                            move(selectedRoad);
                            break;
                        }
                    }
                }
            }
        } else {
            move(world.refuges);
        }
    }

    protected void carry() throws ActionCommandException {
        if (mlpos() instanceof Refuge) {
            unload();
        } else {
            move(world.refuges);
        }
    }

    private void checkBloackade() throws ActionCommandException {

        if ((self().motionlessPosition() instanceof Road) &&
            ((Road) self().motionlessPosition()).passableLinesToHead() == 0 &&
            ((Road) self().motionlessPosition()).passableLinesToTail() == 0 &&
            isNear(world.nearPolice))

        {
            move(world.buildings);
        }
    }


    protected boolean isNear(ArrayList polices) {
        for (Iterator itr12 = polices.iterator(); itr12.hasNext(); ) {
            Humanoid police = (Humanoid) itr12.next();
            if (((MotionlessObject) self().motionlessPosition()).equals(police.
                    motionlessPosition()) ||
                ((MotionlessObject) self().motionlessPosition()).neighborhood().
                contains(police.motionlessPosition())) {
                return true;
            } else {
                Iterator itr3 = ((MotionlessObject) self().motionlessPosition()).
                                neighborhood().iterator();
                if (((MotionlessObject) itr3.next()).neighborhood().contains(
                        police.motionlessPosition()) ||
                    (itr3.hasNext() &&
                     ((MotionlessObject) itr3.next()).
                     neighborhood().contains(police.motionlessPosition()))
                    ||
                    (itr3.hasNext() &&
                     ((MotionlessObject) itr3.next()).neighborhood().
                     contains(police.motionlessPosition())) ||
                    (itr3.hasNext() &&
                     ((MotionlessObject) itr3.next()).
                     neighborhood().contains(police.motionlessPosition()))) {
                    return true;
                }
            }
        }
        return false;
    }


    protected void searchTheInjured(ArrayList reportedInj) throws
            ActionCommandException {
        if (reportedInj == reportedAgents && reportedInj.size() > 0) {
            recordHuman bestFirebrigade = new recordHuman();
            recordHuman temprecHu = (recordHuman) reportedInj.iterator().next();
            if (temprecHu.self == null && temprecHu.id != 0) {
                temprecHu.self = (Humanoid) world.get(temprecHu.id);
            }
            int firePriority = temprecHu.buridness;

            bestFirebrigade = (recordHuman) reportedInj.iterator().next();
            if (bestFirebrigade.self == null && bestFirebrigade.id != 0) {
                bestFirebrigade.self = (Humanoid) world.get(bestFirebrigade.id);
            }
            for (Iterator firitr = reportedInj.iterator(); firitr.hasNext(); ) {
                recordHuman tempFire = (recordHuman) firitr.next();
                if (tempFire.self == null && tempFire.id != 0) {
                    tempFire.self = (Humanoid) world.get(tempFire.id);
                }

                if (tempFire.buridness < firePriority) {
                    bestFirebrigade = tempFire;
                    firePriority = tempFire.buridness;
                }
            }
            bestHumanoid = bestFirebrigade;
            bestHumanoid.setSelected();
            searchTime = world.time();
            return;

        } else {
            if (reportedInj.size() > 1) {
                int prioritySwap;
                recordHuman humanSwap;
                int maxpriority1;
                int maxpriority2;
                recordHuman bestDamage1;
                recordHuman bestDamage2;
                recordHuman checkHuman;
                checkHuman = ((recordHuman) reportedInj.get(0));

                if (checkHuman.self == null && checkHuman.id != 0) {
                    checkHuman.self = (Humanoid) world.get(checkHuman.id);
                }

                maxpriority1 = checkHuman.priority(world.time(),
                        world.ambulanceTeams.size());
                checkHuman = ((recordHuman) reportedInj.get(1));

                if (checkHuman.self == null && checkHuman.id != 0) {
                    checkHuman.self = (Humanoid) world.get(checkHuman.id);
                }

                maxpriority2 = checkHuman.priority(world.time(),
                        world.ambulanceTeams.size());
                prioritySwap = maxpriority1;
                bestDamage1 = (maxpriority1 < maxpriority2) ?
                              (recordHuman) reportedInj.get(0) :
                              (recordHuman) reportedInj.get(1);

                if (bestDamage1.self == null && bestDamage1.id != 0) {
                    bestDamage1.self = (Humanoid) world.get(bestDamage1.id);
                }

                bestDamage2 = (maxpriority1 > maxpriority2) ?
                              (recordHuman) reportedInj.get(0) :
                              (recordHuman) reportedInj.get(1);

                if (bestDamage2.self == null && bestDamage2.id != 0) {
                    bestDamage2.self = (Humanoid) world.get(bestDamage2.id);
                }

                maxpriority1 = (maxpriority1 < maxpriority2) ? maxpriority1 :
                               maxpriority2;
                maxpriority2 = (prioritySwap > maxpriority2) ? prioritySwap :
                               maxpriority2;
                for (Iterator itr = reportedInj.iterator(); itr.hasNext(); ) {
                    recordHuman temp = (recordHuman) itr.next();

                    if (temp.self == null && temp.id != 0) {
                        temp.self = (Humanoid) world.get(temp.id);
                    }

                    if (temp.hp -
                        temp.damage *
                        (world.time() + 3 - temp.time + temp.buridness) > 0) {
                        int priorityTemp = temp.priority(world.time(),
                                world.ambulanceTeams.size());

                        if ((priorityTemp < maxpriority1) &&
                            (temp.mlpos instanceof Building) &&
                            !(temp.mlpos instanceof Refuge) &&
                            !((Building) temp.mlpos).isBurning()) {

                            prioritySwap = maxpriority1;
                            humanSwap = bestDamage1;
                            maxpriority1 = priorityTemp;
                            bestDamage1 = temp;
                            maxpriority2 = (maxpriority2 > prioritySwap) ?
                                           prioritySwap : maxpriority2;
                            bestDamage2 = (maxpriority2 > prioritySwap) ?
                                          humanSwap : bestDamage2;

                        } else if (priorityTemp < maxpriority2 &&
                                   (temp.mlpos instanceof Building) &&
                                   !(temp.mlpos instanceof Refuge) &&
                                   !((Building) temp.mlpos).isBurning()) {
                            maxpriority2 = priorityTemp;
                            bestDamage2 = temp;
                        }
                    }
                }

                bestHumanoid = bestDamage1;
                bestHumanoid.setSelected();
                searchTime = world.time();

            } else if (!reportedInj.isEmpty()) {
                bestHumanoid.setSelected();

                bestHumanoid = ((recordHuman) reportedInj.get(0));
                if (bestHumanoid.self == null && bestHumanoid.id != 0) {
                    bestHumanoid.self = (Humanoid) world.get(bestHumanoid.id);
                }

                searchTime = world.time();
            }
        }
    }


    protected void reportBlockades() {

        if (time() > 6) {
            if (self().motionlessPosition() instanceof Road &&
                ((Road) self().motionlessPosition()).passableLinesToHead() == 0 &&
                ((Road) self().motionlessPosition()).passableLinesToTail() == 0) {

                Road tempRoad = (Road) world.get(self().motionlessPosition().id);
                if (tempRoad.directiontotail == 1) {
                    tempRoad.incEngagedToTail(2);
                } else {
                    tempRoad.incEngagedToHead(2);
                }

                if (!reportedBlockadeOthers.contains(self().motionlessPosition())) {
                    tell("nmv " + self().motionlessPosition().id);
                }

            } else
            if (self().motionlessPosition() instanceof Road &&
                ((Road) self().motionlessPosition()).passableLines() == 0) {
                Road tempRoad = (Road) world.get(self().motionlessPosition().id);
                if (tempRoad.directiontotail == 1) {
                    tempRoad.incEngagedToTail(2);
                } else {
                    tempRoad.incEngagedToHead(2);
                }
                if (!reportedBlockadeOthers.contains(self().motionlessPosition())) {
                    tell("clr " + self().motionlessPosition().id);
                }
            }
        }

    }


    protected void correctZone(int numberOfZoneIn, int zoneIn) {
        int numberOfColumn = numberOfZoneIn / 2 + numberOfZoneIn % 2;
        int column = (zoneIn <= numberOfColumn) ? zoneIn :
                     zoneIn - numberOfColumn;
        int row = (zoneIn > numberOfColumn) ? 1 : 0;
        if (row == 1 && numberOfZoneIn % 2 == 1) {
            numberOfColumn = numberOfColumn - 1;
        }
        long maxYTemp = (row == 0) ?
                        world.minY() + ((world.maxY() - world.minY()) / 2) +
                        800 : world.maxY();
        long maxXTemp = world.minX() +
                        (column * (world.maxX() - world.minX()) /
                         numberOfColumn) +
                        650;
        long minXTemp = world.minX() +
                        ((column - 1) * (world.maxX() - world.minX()) /
                         numberOfColumn) - 650;
        long minYTemp = (row == 0) ? world.minY() :
                        world.minY() + ((world.maxY() - world.minY()) / 2) -
                        800;
        Iterator itr1 = world.roads.iterator();
        Road road = (Road) world.roads.iterator().next();
        for (itr1 = world.roads.iterator(); itr1.hasNext(); ) {
            road = (Road) itr1.next();
            if (road.x() >= minXTemp && road.x() <= maxXTemp &&
                road.y() <= maxYTemp && road.y() >= minYTemp) {
                world.updateBlockByAnother(road);
            }

        }

    }


    protected void hear(RealObject sender, String message) {
        if (sender instanceof AmbulanceCenter) {
            StringTokenizer stFirst = new StringTokenizer(message, "*");
            stFirst.nextToken();
            for (; stFirst.hasMoreTokens(); ) {
                String messageTemp = stFirst.nextToken();

                StringTokenizer st = new StringTokenizer(messageTemp);
                String command = st.nextToken();
                if (command.equals("res")) {
                    int ID = 0;
                    if (st.hasMoreTokens()) {
                        ID = Integer.parseInt(st.nextToken());
                    }
                    ArrayList tempArray = new ArrayList();
                    for (Iterator it = reportedInj.iterator(); it.hasNext(); ) {
                        recordHuman tempRecord = (recordHuman) it.next();

                        if ((tempRecord).id == ID) {
                            tempArray.add(tempRecord);
                        }
                    }

                    reportedInj.remove(Integer.parseInt(st.nextToken()));
                    if (bestHumanoid.id == ID) {
                        bestHumanoid.clearSelected();
                        bestHumanoid = new recordHuman();
                    }
                }

                if ((command.equals("fbh") || command.equals("pfh")) &&
                    st.hasMoreTokens()) {
                    recordHuman damaged = new recordHuman();
                    damaged.time = time() - 2;
                    if (st.hasMoreTokens()) {
                        damaged.hp = Integer.parseInt(st.nextToken());
                    }
                    if (st.hasMoreTokens()) {
                        damaged.damage = Integer.parseInt(st.nextToken());
                    }
                    if (st.hasMoreTokens()) {
                        damaged.buridness = Integer.parseInt(st.nextToken());
                    }
                    if (st.hasMoreTokens()) {
                        damaged.id = Integer.parseInt(st.nextToken());
                        damaged.self = (Humanoid) world.get(damaged.id);

                        if (st.hasMoreTokens()) {
                            damaged.mlpos = (MotionlessObject) world.get(
                                    Integer.parseInt(st.nextToken()));
                            if (!(damaged.mlpos() instanceof Refuge) &&
                                (damaged.mlpos() instanceof Building) &&
                                !damaged.mlpos().equals(mlpos())) {
                                ArrayList tempList = new ArrayList();
                                for (Iterator itr = reportedAgents.iterator();
                                        itr.hasNext(); ) {
                                    recordHuman temp = (recordHuman) itr.next();
                                    if ((temp).mlpos.equals(damaged.mlpos)) {
                                        tempList.add(temp);
                                    }
                                }
                                reportedAgents.removeAll(tempList);
                                reportedAgents.add(damaged);
                            }
                        }
                    }

                }

                if (command.equals("c") && st.hasMoreTokens()) {
                    recordHuman damaged = new recordHuman();
                    damaged.time = time();
                    if (st.hasMoreTokens()) {
                        damaged.hp = Integer.parseInt(st.nextToken());
                    }
                    if (st.hasMoreTokens()) {
                        damaged.damage = Integer.parseInt(st.nextToken());
                    }

                    if (st.hasMoreTokens()) {
                        damaged.buridness = Integer.parseInt(st.nextToken());
                    }
                    if (st.hasMoreTokens()) {
                        damaged.id = Integer.parseInt(st.nextToken());
                        damaged.self = (Humanoid) world.get(damaged.id);
                        if (damaged.self != null && st.hasMoreTokens()) {
                            ArrayList tempList = new ArrayList();
                            damaged.mlpos = (MotionlessObject) world.get(
                                    Integer.parseInt(st.nextToken()));
                            if (!(damaged.mlpos() instanceof Refuge) &&
                                (damaged.mlpos() instanceof Building) &&
                                !(damaged.mlpos().equals(mlpos()))) {
                                for (Iterator itr = reportedInj.iterator();
                                        itr.hasNext(); ) {
                                    recordHuman temp = (recordHuman) itr.next();
                                    if ((temp).mlpos.equals(damaged.mlpos)) {
                                        tempList.add(temp);
                                    }
                                }
                                reportedInj.removeAll(tempList);

                                reportedInj.add(damaged);
                                reportedTillNow.add(damaged);
                            }
                        }
                    }

                }

                if (command.equals("clearZone:") && st.hasMoreTokens()) {
                    int numberOfZoneIn = Integer.parseInt(st.nextToken());
                    int zoneIn = Integer.parseInt(st.nextToken());
                    correctZone(numberOfZoneIn, zoneIn);
                }
                if (command.equals("die") && st.hasMoreTokens()) {
                    dieCommand(st.nextToken());
                }

                if ((command.equals("clr") || command.equals("nmv")) &&
                    st.hasMoreTokens()) {
                    MotionlessObject tempRoad = (MotionlessObject) world.get(
                            Integer.parseInt(st.nextToken()));
                    reportedBlockadeOthers.add(tempRoad);
                    if (tempRoad instanceof Road) {
                        ((Road) tempRoad).setBlock(((Road) tempRoad).width());
                    }
                }
            }
        }

    }

    protected void dieCommand(String dieID) {
        ArrayList removeTemp = new ArrayList();
        for (Iterator itrinj = reportedInj.iterator();
                               itrinj.hasNext(); ) {
            recordHuman currentRecord = (recordHuman) itrinj.next();
            if (currentRecord.id == Integer.parseInt(dieID)) {
                removeTemp.add(currentRecord);
            }
        }

        reportedInj.removeAll(removeTemp);

    }

}
