// Copyright (C) 2002 Takeshi Morimoto <morimoto@takopen.cs.uec.ac.jp>
// All rights reserved.

package yab.agent;

import java.net.*;
import java.util.*;
import yab.agent.*;
import yab.agent.object.*;

public abstract class HumanoidAgent extends Agent {
    protected HumanoidAgent(int agentType, InetAddress address, int port) {
        super(agentType, address, port);
    }

    private Humanoid self() { return (Humanoid) world.self; }
    private long  priviousEnd;
    private int lastTime;
    public  int[] lastRoute=new int[100];
    protected int hearingLimit() { return HEARING_LIMIT_OF_HUMANOID; }

    protected void move(MotionlessObject destination)
        throws ActionCommandException {
        move(Collections.singleton(destination));
    }
    protected void move(Collection destinations) throws ActionCommandException{
         move(getRoute(destinations));
    }
    
    //000000000000000000000000000000000000000000000000000000000000000000000000000000
    protected void move(Route routePlan) throws ActionCommandException {
                
       //if(lastRoute.length!=0 &&  self().motionlessPosition() instanceof Road &&
         //       ( (Road) self().motionlessPosition()).passableLines()!=0 && 
           //     (routePlan.toIDs()[routePlan.size()-1])==priviousEnd && world.time()==lastTime+1) return false;                
         move(routePlan.toIDs());
         
    }
    
    protected void move(int[] routePlan) throws ActionCommandException {
        socket().akMove(self().id, routePlan);   
         throw new ActionCommandException();
         
    }
    
    public int[] getLast()
    {
        return lastRoute;
    }
  
  
    protected Route getRoute(Collection destinations) {
        MotionlessObject pos = self().motionlessPosition();
        if (!(pos instanceof Road)  ||  ((Road) pos).passableLines() >= 1)
            return Router.get(pos, destinations, RELIABILITY_COST_FUNCTION);
        Route route = Router.get(outsidePosition(), destinations,
                                 RELIABILITY_COST_FUNCTION);
        modifyOriginToCurrentPosition(route);
        return route;
    }
    public MotionlessObject outsidePosition() {
        Road rd = (Road) self().motionlessPosition();
        return (self().positionExtra() < rd.length() / 2)
            ? rd.head() : rd.tail();
    }
    public void modifyOriginToCurrentPosition(Route route) {
        MotionlessObject pos = self().motionlessPosition();
        for (Route rt = route;  true;  rt = rt.previous()) {
            if (rt.end == pos) {
                rt.setPrevious(null);
                return;
            }
            if (rt.previous() == null) {
                rt.setPrevious(new Route(pos));
                return;
            }
        }
    }
    private static final Router.CostFunction RELIABILITY_COST_FUNCTION
      = new Router.CostFunction() {
     public float cost(MotionlessObject from, MotionlessObject to) {
                    final int WEIGHT = 1000;
                    float Cost=0.0f;

                   if (from instanceof PointObject )
                    {
                      Collection near_road= from.neighborhood();

                      for (Iterator j=near_road.iterator();j.hasNext();)
                      {

                          MotionlessObject edge=(MotionlessObject) j.next();
                          if (edge instanceof Road)
                          {
                              Road rd=(Road) edge ;

                              if (to instanceof PointObject)
                              {
                                if (rd.neighborhood().contains(to) )
                                 {

                                  Cost=rd.length()/1000;

                                    if(from==(MotionlessObject)rd.tail()&&
                                                     rd.passableLinesToHead()<1)
                                       Cost=Cost+WEIGHT*WEIGHT;
                                    if(from==(MotionlessObject)rd.head()&&
                                                     rd.passableLinesToTail()<1)
                                       Cost=Cost+WEIGHT*WEIGHT;

                                    return Cost;
                                }
                              }
                              else
                              {
                                 if (rd==(Road)to)
                                  if(from==(MotionlessObject)rd.tail()&&
                                                     rd.passableLinesToHead()<1)
                                       Cost=Cost+WEIGHT*WEIGHT;
                                   if(from==(MotionlessObject)rd.head()&&
                                                     rd.passableLinesToTail()<1)
                                       Cost=Cost+WEIGHT*WEIGHT;
                                    return Cost;
                              }
                          }// is not road
                          else if (edge instanceof PointObject)
                              return Cost;
                          else
                              return Integer.MAX_VALUE*Integer.MAX_VALUE;
                      }

                    }
                   else
                   {
                       Road from1=(Road)from;
                       if (to instanceof PointObject)
                           {
                               if (from1.neighborhood().contains(to) )
                                 {

                                  Cost=from1.length()/1000;
                                    if(to==from1.tail()&&
                                                     from1.passableLinesToTail()<1)
                                       Cost=Cost+WEIGHT*WEIGHT;
                                    if(to==from1.head()&&
                                                     from1.passableLinesToHead()<1)
                                       Cost=Cost+WEIGHT*WEIGHT;
                                    return Cost;
                                }
                             }
                              else
                              {
                                 if(from==to)
                                 {
                                     Road from2=(Road) from;
                                     if( from2.passableLines() <1)
                                             return Integer.MAX_VALUE*Integer.MAX_VALUE;
                                     else return Integer.MAX_VALUE;
                                 }
                                 else return Integer.MAX_VALUE*Integer.MAX_VALUE;
                              }

                   }
                    return Cost;
                   }
    };

}
